@@ -3,6 +3,7 @@
 
 #include <symengine/basic.h>
 #include <symengine/visitor.h>
+#include <float.h>
 
 #ifdef HAVE_SYMENGINE_LLVM
 
@@ -11,6 +12,7 @@ namespace llvm
 {
 class Module;
 class Value;
+class Type;
 class Function;
 class ExecutionEngine;
 class MemoryBufferRef;
@@ -27,7 +29,7 @@ namespace SymEngine
 
 class IRBuilder;
 
-class LLVMDoubleVisitor : public BaseVisitor<LLVMDoubleVisitor>
+class LLVMVisitor : public BaseVisitor<LLVMVisitor>
 {
 protected:
     vec_basic symbols;
@@ -45,6 +47,7 @@ protected:
     llvm::Module *mod;
     std::string membuffer;
     llvm::Function *get_function_type(llvm::LLVMContext *);
+    virtual llvm::Type *get_float_type(llvm::LLVMContext *) = 0;
 
 public:
     llvm::Value *apply(const Basic &b);
@@ -59,16 +62,13 @@ public:
 
     static std::vector<llvm::Pass *> create_default_passes(int optlevel);
 
-    double call(const std::vector<double> &vec);
-    void call(double *outs, const double *inps);
-
     // Helper functions
     void set_double(double d);
     llvm::Function *get_external_function(const std::string &name,
                                           size_t nargs = 1);
     llvm::Function *get_powi();
 
-    void bvisit(const Integer &x, bool as_int32 = false);
+    void bvisit(const Integer &x);
     void bvisit(const Rational &x);
     void bvisit(const RealDouble &x);
 #ifdef HAVE_SYMENGINE_MPFR
@@ -84,21 +84,6 @@ public:
     void bvisit(const Basic &);
     void bvisit(const Sin &x);
     void bvisit(const Cos &x);
-    void bvisit(const Tan &x);
-    void bvisit(const ASin &x);
-    void bvisit(const ACos &x);
-    void bvisit(const ATan &x);
-    void bvisit(const ATan2 &x);
-    void bvisit(const Sinh &x);
-    void bvisit(const Cosh &x);
-    void bvisit(const Tanh &x);
-    void bvisit(const ASinh &x);
-    void bvisit(const ACosh &x);
-    void bvisit(const ATanh &x);
-    void bvisit(const Gamma &x);
-    void bvisit(const LogGamma &x);
-    void bvisit(const Erf &x);
-    void bvisit(const Erfc &x);
     void bvisit(const Piecewise &x);
     void bvisit(const BooleanAtom &x);
     void bvisit(const And &x);
@@ -115,6 +100,7 @@ public:
     void bvisit(const Infty &x);
     void bvisit(const Floor &x);
     void bvisit(const Ceiling &x);
+    void bvisit(const Truncate &x);
     void bvisit(const Sign &x);
     // Return the compiled function as a binary string which can be loaded using
     // `load`
@@ -122,6 +108,86 @@ public:
     // Load a previously compiled function from a string
     void loads(const std::string &s);
 };
-}
+
+class LLVMDoubleVisitor : public LLVMVisitor
+{
+public:
+    double call(const std::vector<double> &vec);
+    void call(double *outs, const double *inps);
+    llvm::Type *get_float_type(llvm::LLVMContext *) override;
+    void visit(const Tan &x) override;
+    void visit(const ASin &x) override;
+    void visit(const ACos &x) override;
+    void visit(const ATan &x) override;
+    void visit(const ATan2 &x) override;
+    void visit(const Sinh &x) override;
+    void visit(const Cosh &x) override;
+    void visit(const Tanh &x) override;
+    void visit(const ASinh &x) override;
+    void visit(const ACosh &x) override;
+    void visit(const ATanh &x) override;
+    void visit(const Gamma &x) override;
+    void visit(const LogGamma &x) override;
+    void visit(const Erf &x) override;
+    void visit(const Erfc &x) override;
+};
+
+class LLVMFloatVisitor : public LLVMVisitor
+{
+public:
+    float call(const std::vector<float> &vec);
+    void call(float *outs, const float *inps);
+    llvm::Type *get_float_type(llvm::LLVMContext *) override;
+    void visit(const Tan &x) override;
+    void visit(const ASin &x) override;
+    void visit(const ACos &x) override;
+    void visit(const ATan &x) override;
+    void visit(const ATan2 &x) override;
+    void visit(const Sinh &x) override;
+    void visit(const Cosh &x) override;
+    void visit(const Tanh &x) override;
+    void visit(const ASinh &x) override;
+    void visit(const ACosh &x) override;
+    void visit(const ATanh &x) override;
+    void visit(const Gamma &x) override;
+    void visit(const LogGamma &x) override;
+    void visit(const Erf &x) override;
+    void visit(const Erfc &x) override;
+};
+
+#if SYMENGINE_SIZEOF_LONG_DOUBLE > 8 && defined(__x86_64__) || defined(__i386__)
+#define SYMENGINE_HAVE_LLVM_LONG_DOUBLE 1
+class LLVMLongDoubleVisitor : public LLVMVisitor
+{
+public:
+    long double call(const std::vector<long double> &vec);
+    void call(long double *outs, const long double *inps);
+    llvm::Type *get_float_type(llvm::LLVMContext *) override;
+    void visit(const Tan &x) override;
+    void visit(const ASin &x) override;
+    void visit(const ACos &x) override;
+    void visit(const ATan &x) override;
+    void visit(const ATan2 &x) override;
+    void visit(const Sinh &x) override;
+    void visit(const Cosh &x) override;
+    void visit(const Tanh &x) override;
+    void visit(const ASinh &x) override;
+    void visit(const ACosh &x) override;
+    void visit(const ATanh &x) override;
+    void visit(const Gamma &x) override;
+    void visit(const LogGamma &x) override;
+    void visit(const Erf &x) override;
+    void visit(const Erfc &x) override;
+    void visit(const Integer &x) override;
+    void visit(const Rational &x) override;
+    void convert_from_mpfr(const Basic &x);
+    void visit(const Constant &x) override;
+#ifdef HAVE_SYMENGINE_MPFR
+    void visit(const RealMPFR &x) override;
+#endif
+};
+#endif
+
+} // namespace SymEngine
 #endif
 #endif // SYMENGINE_LAMBDA_DOUBLE_H
