@@ -2,6 +2,7 @@
 #define SYMENGINE_MATRIX_H
 
 #include <symengine/basic.h>
+#include <symengine/sets.h>
 
 namespace SymEngine
 {
@@ -68,12 +69,20 @@ public:
     // Fraction free LDU factorization
     virtual void FFLDU(MatrixBase &L, MatrixBase &D, MatrixBase &U) const = 0;
 
+    // QR factorization
+    virtual void QR(MatrixBase &Q, MatrixBase &R) const = 0;
+
+    // Cholesky decomposition
+    virtual void cholesky(MatrixBase &L) const = 0;
+
     // Solve Ax = b using LU factorization
     virtual void LU_solve(const MatrixBase &b, MatrixBase &x) const = 0;
 };
 
 typedef std::vector<std::pair<int, int>> permutelist;
 
+class CSRMatrix;
+
 // ----------------------------- Dense Matrix --------------------------------//
 class DenseMatrix : public MatrixBase
 {
@@ -83,6 +92,7 @@ public:
     DenseMatrix(const DenseMatrix &);
     DenseMatrix(unsigned row, unsigned col);
     DenseMatrix(unsigned row, unsigned col, const vec_basic &l);
+    DenseMatrix(const vec_basic &column_elements);
     // Resize
     void resize(unsigned i, unsigned j);
 
@@ -92,6 +102,7 @@ public:
     // Get and set elements
     virtual RCP<const Basic> get(unsigned i, unsigned j) const;
     virtual void set(unsigned i, unsigned j, const RCP<const Basic> &e);
+    virtual vec_basic as_vec_basic() const;
 
     virtual unsigned nrows() const
     {
@@ -144,19 +155,25 @@ public:
     // Fraction free LDU factorization
     virtual void FFLDU(MatrixBase &L, MatrixBase &D, MatrixBase &U) const;
 
+    // QR factorization
+    virtual void QR(MatrixBase &Q, MatrixBase &R) const;
+
+    // Cholesky decomposition
+    virtual void cholesky(MatrixBase &L) const;
+
     // Return the Jacobian of the matrix
     friend void jacobian(const DenseMatrix &A, const DenseMatrix &x,
-                         DenseMatrix &result);
+                         DenseMatrix &result, bool diff_cache);
     // Return the Jacobian of the matrix using sdiff
     friend void sjacobian(const DenseMatrix &A, const DenseMatrix &x,
-                          DenseMatrix &result);
+                          DenseMatrix &result, bool diff_cache);
 
     // Differentiate the matrix element-wise
     friend void diff(const DenseMatrix &A, const RCP<const Symbol> &x,
-                     DenseMatrix &result);
+                     DenseMatrix &result, bool diff_cache);
     // Differentiate the matrix element-wise using SymPy compatible diff
     friend void sdiff(const DenseMatrix &A, const RCP<const Basic> &x,
-                      DenseMatrix &result);
+                      DenseMatrix &result, bool diff_cache);
 
     // Friend functions related to Matrix Operations
     friend void add_dense_dense(const DenseMatrix &A, const DenseMatrix &B,
@@ -172,6 +189,12 @@ public:
                                 unsigned row_start, unsigned col_start,
                                 unsigned row_end, unsigned col_end,
                                 unsigned row_step, unsigned col_step);
+    void row_join(const DenseMatrix &B);
+    void col_join(const DenseMatrix &B);
+    void row_insert(const DenseMatrix &B, unsigned pos);
+    void col_insert(const DenseMatrix &B, unsigned pos);
+    void row_del(unsigned k);
+    void col_del(unsigned k);
 
     // Row operations
     friend void row_exchange_dense(DenseMatrix &A, unsigned i, unsigned j);
@@ -181,6 +204,9 @@ public:
                                   RCP<const Basic> &c);
     friend void permuteFwd(DenseMatrix &A, permutelist &pl);
 
+    // Column operations
+    friend void column_exchange_dense(DenseMatrix &A, unsigned i, unsigned j);
+
     // Gaussian elimination
     friend void pivoted_gaussian_elimination(const DenseMatrix &A,
                                              DenseMatrix &B,
@@ -198,6 +224,10 @@ public:
         const DenseMatrix &A, DenseMatrix &B, permutelist &pivotlist);
     friend unsigned pivot(DenseMatrix &B, unsigned r, unsigned c);
 
+    friend void reduced_row_echelon_form(const DenseMatrix &A, DenseMatrix &B,
+                                         vec_uint &pivot_cols,
+                                         bool normalize_last);
+
     // Ax = b
     friend void diagonal_solve(const DenseMatrix &A, const DenseMatrix &b,
                                DenseMatrix &x);
@@ -236,14 +266,22 @@ public:
     // Inverse
     friend void inverse_fraction_free_LU(const DenseMatrix &A, DenseMatrix &B);
     friend void inverse_LU(const DenseMatrix &A, DenseMatrix &B);
+    friend void inverse_pivoted_LU(const DenseMatrix &A, DenseMatrix &B);
     friend void inverse_gauss_jordan(const DenseMatrix &A, DenseMatrix &B);
 
+    // Vector-specific methods
+    friend void dot(const DenseMatrix &A, const DenseMatrix &B, DenseMatrix &C);
+    friend void cross(const DenseMatrix &A, const DenseMatrix &B,
+                      DenseMatrix &C);
+
     // NumPy-like functions
     friend void eye(DenseMatrix &A, int k);
     friend void diag(DenseMatrix &A, vec_basic &v, int k);
     friend void ones(DenseMatrix &A);
     friend void zeros(DenseMatrix &A);
 
+    friend CSRMatrix;
+
 private:
     // Matrix elements are stored in row-major order
     vec_basic m_;
@@ -258,8 +296,14 @@ class CSRMatrix : public MatrixBase
 public:
     CSRMatrix();
     CSRMatrix(unsigned row, unsigned col);
+    CSRMatrix(unsigned row, unsigned col, const std::vector<unsigned> &p,
+              const std::vector<unsigned> &j, const vec_basic &x);
     CSRMatrix(unsigned row, unsigned col, std::vector<unsigned> &&p,
               std::vector<unsigned> &&j, vec_basic &&x);
+    CSRMatrix &operator=(CSRMatrix &&other);
+    CSRMatrix(const CSRMatrix &) = default;
+    std::tuple<std::vector<unsigned>, std::vector<unsigned>, vec_basic>
+    as_vectors() const;
 
     bool is_canonical() const;
 
@@ -297,6 +341,7 @@ public:
 
     // Matrix transpose
     virtual void transpose(MatrixBase &result) const;
+    CSRMatrix transpose() const;
 
     // Extract out a submatrix
     virtual void submatrix(MatrixBase &result, unsigned row_start,
@@ -319,6 +364,12 @@ public:
     // Fraction free LDU factorization
     virtual void FFLDU(MatrixBase &L, MatrixBase &D, MatrixBase &U) const;
 
+    // QR factorization
+    virtual void QR(MatrixBase &Q, MatrixBase &R) const;
+
+    // Cholesky decomposition
+    virtual void cholesky(MatrixBase &L) const;
+
     static void csr_sum_duplicates(std::vector<unsigned> &p_,
                                    std::vector<unsigned> &j_, vec_basic &x_,
                                    unsigned row_);
@@ -343,6 +394,10 @@ public:
                               const std::vector<unsigned> &i,
                               const std::vector<unsigned> &j,
                               const vec_basic &x);
+    static CSRMatrix jacobian(const vec_basic &exprs, const vec_sym &x,
+                              bool diff_cache = true);
+    static CSRMatrix jacobian(const DenseMatrix &A, const DenseMatrix &x,
+                              bool diff_cache = true);
 
     friend void csr_matmat_pass1(const CSRMatrix &A, const CSRMatrix &B,
                                  CSRMatrix &C);
@@ -354,8 +409,8 @@ public:
 
     friend void csr_binop_csr_canonical(
         const CSRMatrix &A, const CSRMatrix &B, CSRMatrix &C,
-        RCP<const Basic>(&bin_op)(const RCP<const Basic> &,
-                                  const RCP<const Basic> &));
+        RCP<const Basic> (&bin_op)(const RCP<const Basic> &,
+                                   const RCP<const Basic> &));
 
 private:
     std::vector<unsigned> p_;
@@ -367,26 +422,42 @@ private:
 };
 
 // Return the Jacobian of the matrix
-void jacobian(const DenseMatrix &A, const DenseMatrix &x, DenseMatrix &result);
+void jacobian(const DenseMatrix &A, const DenseMatrix &x, DenseMatrix &result,
+              bool diff_cache = true);
 // Return the Jacobian of the matrix using sdiff
-void sjacobian(const DenseMatrix &A, const DenseMatrix &x, DenseMatrix &result);
+void sjacobian(const DenseMatrix &A, const DenseMatrix &x, DenseMatrix &result,
+               bool diff_cache = true);
 
 // Differentiate all the elements
-void diff(const DenseMatrix &A, const RCP<const Symbol> &x,
-          DenseMatrix &result);
+void diff(const DenseMatrix &A, const RCP<const Symbol> &x, DenseMatrix &result,
+          bool diff_cache = true);
 // Differentiate all the elements using SymPy compatible diff
-void sdiff(const DenseMatrix &A, const RCP<const Basic> &x,
-           DenseMatrix &result);
+void sdiff(const DenseMatrix &A, const RCP<const Basic> &x, DenseMatrix &result,
+           bool diff_cache = true);
 
 // Get submatrix from a DenseMatrix
 void submatrix_dense(const DenseMatrix &A, DenseMatrix &B, unsigned row_start,
                      unsigned col_start, unsigned row_end, unsigned col_end,
                      unsigned row_step = 1, unsigned col_step = 1);
 
+// Row operations
+void row_exchange_dense(DenseMatrix &A, unsigned i, unsigned j);
+void row_mul_scalar_dense(DenseMatrix &A, unsigned i, RCP<const Basic> &c);
+void row_add_row_dense(DenseMatrix &A, unsigned i, unsigned j,
+                       RCP<const Basic> &c);
+
+// Column operations
+void column_exchange_dense(DenseMatrix &A, unsigned i, unsigned j);
+
+// Vector-specific methods
+void dot(const DenseMatrix &A, const DenseMatrix &B, DenseMatrix &C);
+void cross(const DenseMatrix &A, const DenseMatrix &B, DenseMatrix &C);
+
 // Matrix Factorization
 void LU(const DenseMatrix &A, DenseMatrix &L, DenseMatrix &U);
-
 void LDL(const DenseMatrix &A, DenseMatrix &L, DenseMatrix &D);
+void QR(const DenseMatrix &A, DenseMatrix &Q, DenseMatrix &R);
+void cholesky(const DenseMatrix &A, DenseMatrix &L);
 
 // Inverse
 void inverse_fraction_free_LU(const DenseMatrix &A, DenseMatrix &B);
@@ -414,6 +485,9 @@ RCP<const Basic> det_berkowitz(const DenseMatrix &A);
 // then the corresponding polynomial is `x**2 - 2x + 3`.
 void char_poly(const DenseMatrix &A, DenseMatrix &B);
 
+// returns a finiteset of eigenvalues of a matrix
+RCP<const Set> eigen_values(const DenseMatrix &A);
+
 // Mimic `eye` function in NumPy
 void eye(DenseMatrix &A, int k = 0);
 
@@ -426,6 +500,11 @@ void ones(DenseMatrix &A);
 // Create a matrix filled with zeros
 void zeros(DenseMatrix &A);
 
+// Reduced row echelon form and returns the cols with pivots
+void reduced_row_echelon_form(const DenseMatrix &A, DenseMatrix &B,
+                              vec_uint &pivot_cols,
+                              bool normalize_last = false);
+
 // Returns true if `b` is exactly the type T.
 // Here T can be a DenseMatrix, CSRMatrix, etc.
 template <class T>
